"""JARVIS bot utility commands."""
import logging
import platform
from io import BytesIO

import psutil
from aiofile import AIOFile, LineReader
from dis_snek import MessageContext, Scale, Snake
from dis_snek.models.discord.embed import EmbedField
from dis_snek.models.discord.file import File
from molter import msg_command

from jarvis.utils import build_embed


class BotutilCog(Scale):
    """JARVIS Bot Utility Cog."""

    def __init__(self, bot: Snake):
        self.bot = bot
        self.logger = logging.getLogger(__name__)
        self.add_scale_check(self.is_owner)

    async def is_owner(self, ctx: MessageContext) -> bool:
        """Checks if author is bot owner."""
        return ctx.author.id == self.bot.owner.id

    @msg_command(name="tail")
    async def _tail(self, ctx: MessageContext, count: int = 10) -> None:
        lines = []
        async with AIOFile("jarvis.log", "r") as af:
            async for line in LineReader(af):
                lines.append(line)
                if len(lines) == count + 1:
                    lines.pop(0)
        log = "".join(lines)
        if len(log) > 1500:
            with BytesIO() as file_bytes:
                file_bytes.write(log.encode("UTF8"))
                file_bytes.seek(0)
                log = File(file_bytes, file_name=f"tail_{count}.log")
                await ctx.reply(content=f"Here's the last {count} lines of the log", file=log)
        else:
            await ctx.reply(content=f"```\n{log}\n```")

    @msg_command(name="log")
    async def _log(self, ctx: MessageContext) -> None:
        async with AIOFile("jarvis.log", "r") as af:
            with BytesIO() as file_bytes:
                raw = await af.read_bytes()
                file_bytes.write(raw)
                file_bytes.seek(0)
                log = File(file_bytes, file_name="jarvis.log")
                await ctx.reply(content="Here's the latest log", file=log)

    @msg_command(name="crash")
    async def _crash(self, ctx: MessageContext) -> None:
        raise Exception("As you wish")

    @msg_command(name="sysinfo")
    async def _sysinfo(self, ctx: MessageContext) -> None:
        st_ts = int(self.bot.start_time.timestamp())
        ut_ts = int(psutil.boot_time())
        fields = [
            EmbedField(name="Operation System", value=platform.system() or "Unknown", inline=False),
            EmbedField(name="Version", value=platform.release() or "N/A", inline=False),
            EmbedField(name="System Start Time", value=f"<t:{ut_ts}:F> (<t:{ut_ts}:R>)"),
            EmbedField(name="Python Version", value=platform.python_version()),
            EmbedField(name="Bot Start Time", value=f"<t:{st_ts}:F> (<t:{st_ts}:R>)"),
        ]
        embed = build_embed(title="System Info", description="", fields=fields)
        await ctx.send(embed=embed)


def setup(bot: Snake) -> None:
    """Add BotutilCog to JARVIS"""
    BotutilCog(bot)
